<?php

namespace TDevAgency\FilesManager;

use claviska\SimpleImage;
use Illuminate\Support\Facades\View;
use Illuminate\Support\Str;
use TDevAgency\FilesManager\Helpers\CacheHelper;
use TDevAgency\FilesManager\ResourceModels\TdfmFile;

trait MediaHelper
{
    /** @var bool */

    private $forceWebP = false;

    public function getImage(int $id, ?string $size = null, bool $webp = false): string
    {
        return asset($this->getImagePath($id, $size, $webp));
    }

    private function getImagePath(int $id, ?string $size = null, bool $webp = false): ?string
    {
        /** @var TdfmFile $filesManager */
        $filesManager = app(CacheHelper::class)->getFileData($id);

        if ($filesManager === null) {
            return null;
        }

        $isSvg = strpos($filesManager->name, '.svg') !== false;
        if ($this->forceWebP) {
            $webpSupport = true;
        } else {
            $webpSupport = strpos(request()->headers->get('accept'), 'image/webp') !== false;
        }
        if ($webp && $webpSupport && !$isSvg) {
            $filesManager->original_name = $this->getWebpName($filesManager->original_name);
        }

        if ($size === null || $isSvg) {
            $path = 'media/' . $id . '/' . $filesManager->original_name;
        } else {
            $path = 'media/' . $id . '/' . $size . '/' . $filesManager->original_name;
        }

        return $path;
    }

    /**
     * @param $file
     * @return string|null
     */
    private function getWebpName($file): ?string
    {
        $pathInfo = pathinfo($file);

        if (in_array(Str::lower($pathInfo['extension']), ['jpg', 'jpeg', 'png', 'gif'])) {
            return $pathInfo['filename'] . '.webp';
        }

        return null;
    }

    public function getImageWidth($id, $size = null): ?int
    {
        $closure = function () use ($id, $size) {
            $path = $this->getImagePath($id, $size);

            if ($path === null) {
                return null;
            }

            if (strpos($path, '.svg') !== false) {
                return null;
            }

            return (new SimpleImage())->fromFile($path)->getWidth();
        };

        return app(CacheHelper::class)->rememberSideSize($id, $size, $closure);
    }

    public function getImageHeight($id, $size = null): ?int
    {
        $closure = function () use ($id, $size) {
            $path = $this->getImagePath($id, $size);

            if ($path === null) {
                return null;
            }

            if (strpos($path, '.svg') !== false) {
                return null;
            }
            return (new SimpleImage())->fromFile($path)->getHeight();
        };

        return app(CacheHelper::class)->rememberSideSize($id, $size, $closure, 'height');
    }

    public function getPictureTag($id, $size = null): ?string
    {
        if ($id === null) {
            return null;
        }
        $size = $size ?? 'xl';

        /**
         * @var $filesManager TdfmFile
         */
        $filesManager = app(CacheHelper::class)->getFileData($id);
        if ($filesManager === null) {
            return null;
        }

        $filesManager->webpName = $this->getWebpName($filesManager->original_name);
        $tag = [
            'sources' => [],
            'src' => preg_match('/^(\w+)(\.)(svg)+$/', $filesManager->original_name) ? asset(
                'media/original/' . $filesManager->original_name
            ) :
                asset('media/' . $id . '/' . $size . '/' . $filesManager->original_name)
        ];
        if (preg_match('/^image\/(jpe?g|gif|png)+$/', $filesManager->file_mime)) {
            $tag['sources'][] = [
                'src' => asset('media/' . $id . '/' . $size . '/' . $filesManager->webpName),
                'mime' => 'image/webp',
            ];
            $tag['sources'][] = [
                'src' => asset('media/' . $id . '/' . $size . '/' . $filesManager->original_name),
                'mime' => $filesManager->file_mime,
            ];
        }

        return View::make('files-manager::picture')->with('picture', $tag)->render();
    }

    protected function forceSupportWebp(bool $force = true)
    {
        $this->forceWebP = $force;
        return $this;
    }
}
